/****************************************************************************
 Include Section
*****************************************************************************/

#include "global.h"

/*****************************************************************************
 Symbolic constants / definitions
******************************************************************************/

#define SPI0CLK         PE0
#define SPI0MISO        PE1
#define SPI0MOSI        PE2

/*****************************************************************************
 Global Variables
******************************************************************************/

.EXTERN loader_file;

/*****************************************************************************
 Variables
******************************************************************************/

.SECTION L1_data;
.ALIGN 4;

/*****************************************************************************
 Functions
******************************************************************************/

.SECTION L1_code;

/*****************************************************************************
 Prototypes
******************************************************************************/

.GLOBAL __spi_hostboot_run;

//.GLOBAL gpio_ports_setup;
//.GLOBAL spi_setup;
//.GLOBAL spi_putc;
//.GLOBAL spi_disable;
//.GLOBAL spi_calc_baud;


/****************************************************************************
 Function:              __spi_hostboot_run
 Description:           do the transfer
 Input Parameters:      none
 Return Parameters:     none
 Registers Used:        R7,P5:3,ASTAT
 Global Registers Used: none
 C-Callable :           no
*****************************************************************************/

__spi_hostboot_run:

    link 0;
    [--SP] = ASTAT;
    [--SP] = (R7:6,P5:3);
    PUSH_LOOP_CNTx(0);
    IMM32(P5,SYS_MMR_BASE);


    call gpio_ports_setup;

    /* wait for slave to deassert host wait signal */
    initial.wait:
    R7 = w[P5 + lo(PORTE)] (z);
    CC = bittst(R7,bitpos(PFHW));
    if CC jump initial.wait;

    call spi_setup;

    /* This is our pointer to the LDR file */
    IMM32(P4,loader_file);
    IMM32(P3,length(loader_file));

    /* assert CS low (PFSSEL) */
    R7 = PFSSEL(z);
    w[P5 + lo(PORTE_CLEAR)] = R7;


    lsetup(spi_hostboot_run.loop_start, spi_hostboot_run.loop_end) LC0 = P3;
    spi_hostboot_run.loop_start:    R7 = w[P5 + lo(PORTE)] (z);
                                    CC = bittst(R7,bitpos(PFHW));
                                    if CC jump spi_hostboot_run.loop_start;

                                    R7 = b[P4++](z) ;
                                    w[P5 + lo(SPI0_TDBR)] = R7;
    spi_hostboot_run.loop_wait:     R7 = w[P5 + lo(SPI0_STAT)] (z);
                                    CC = bittst(R7,bitpos(RXS));
                                    if !CC jump spi_hostboot_run.loop_wait;       
    spi_hostboot_run.loop_end:      R7 = w[P5 + lo(SPI0_RDBR)] (z);


    /* assert CS high (PFSSEL) */
    R7 = PFSSEL(z);
    w[P5 + lo(PORTE_SET)] = R7;


    call spi_disable;


    POP_LOOP_CNTx(0);
    (R7:6,P5:3) = [SP++];
    ASTAT = [SP++];
    unlink;

    rts;
__spi_hostboot_run.END:


/*****************************************************************************
 Function:              spi_putc
 Description:           run a SPI tranmission -> transmit SPI
                        Hardware Loop used
                        No Interrupts
 Input Parameters:      R0
 Return Parameters:     none
 Registers Used:        R7:6,ASTAT
 Global Registers Used: P5 -> assumed to contain SYS_MMR_BASE address
 C-Callable :           no
*****************************************************************************/

spi_putc:

    link 0;
    [--SP] = ASTAT;
    [--SP] = (R7:6);

    /* Write to SPI_TDBR */
    w[P5 + lo(SPI0_TDBR)] = R0;
    /* Dummy read of SPI_RDBR kicks off transfer */
    R7 = w[P5 + lo(SPI0_RDBR)] (z);
    spi_putc.check_txs:     R7 = w[P5 + lo(SPI0_STAT)] (z);
                            CC = bittst (R7,bitpos(TXS));
                            if CC jump spi_putc.check_txs;

    /* Poll for SPIF = 1 (SPI finished) */
    spi_putc.check_spif:    R7 = w[P5 + lo(SPI0_STAT)] (z);
                            CC = bittst (R7,bitpos(SPIF));
                            if !CC jump spi_putc.check_spif;

    (R7:6) = [SP++];
    ASTAT = [SP++];
    unlink;

    rts;
spi_putc.end:


/****************************************************************************
 Function:              gpio_ports_setup
 Description:           setup up GP ports
 Input Parameters:      none
 Return Parameters:     none
 Registers Used:        R7:6,P5
 Global Registers Used: none
 C-Callable :           no
*****************************************************************************/

gpio_ports_setup:

    link 0;
    [--SP] = (R7:6,P5:5);
    IMM32(P5,SYS_MMR_BASE);


    R7 = w[P5 + lo(PORTE_FER)];
    R6 = ( SPI0CLK | SPI0MISO | SPI0MOSI ) (z);
    R7 = R7 | R6;
    w[P5 + lo(PORTE_FER)] = R7;

    /* enable SPI0SCK, SPI0MISO and SPI0MOSI mux mode */
    R7 = [P5 + lo(PORTE_MUX)];
    IMM32(R6,MUX(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0));
    R7 = R7 | R6;
    [P5 + lo(PORTE_MUX)] = R7;

    R7 = PFHW;
    w[P5 + lo(PORTE_DIR_CLEAR)] = R7;
    R7 = PFSSEL;
    w[P5 + lo(PORTE_DIR_SET)] = R7;

    R7 = PFSSEL(z);
    w[P5 + lo(PORTE_SET)] = R7;

    /* Enable PF */
    R7 = w[P5 + lo(PORTE_INEN)](z);
    bitset(R7,bitpos(PFHW));
    w[P5 + lo(PORTE_INEN)] = R7;


    (R7:6,P5:5) = [SP++];
    unlink;

    rts;
gpio_ports_setup.end:


/****************************************************************************
 Function:              spi_setup
 Description:           generic SPI setup up function
 Input Parameters:      none
 Return Parameters:     none
 Registers Used:        R7,R0,P5
 Global Registers Used: none
 C-Callable :           no
*****************************************************************************/

spi_setup:

    link 0;
    [--SP] = R7;
    [--SP] = R0;
    [--SP] = P5;
    IMM32(P5,SYS_MMR_BASE);


    call spi_calc_baud;
    w[P5 + lo(SPI0_BAUD)] = R0; 

    /* enable SPI, nondma tx mode, 8bit */
    R7 =    TDBR_CORE   |   /* TDBR Write Initiates, IRQ when TDBR Empty */
           nSZ          |   /* Send Zero */
           nGM          |   /* Get More Data */
           nPSSE        |   /* Slave-Select Enable */
           nEMISO       |   /* Enable MISO Output */
           nSIZE        |   /* Size of Words */
           nLSBF        |   /* LSB First */
            CPHA        |   /* Clock Phase */
           nCPOL        |   /* Clock Polarity */
            MSTR        |   /* HOST Mode */
           nWOM         |   /* Write Open Drain HOST */
            SPE         (z);/* SPI Enable */
    w[P5 + lo(SPI0_CTL)] = R7;


    P5 = [SP++];
    R0 = [SP++];
    R7 = [SP++];
    unlink;

    rts;
spi_setup.end:


/****************************************************************************
 Function:              spi_calc_baud
 Description:           generic SPI baud rate calculation function
 Input Parameters:      none
 Return Parameters:     R0
 Registers Used:        R1,R0
 Global Registers Used: none
 C-Callable :           no
*****************************************************************************/

spi_calc_baud:

    link 0;
    [--SP] = R1;

    call __get_sclk_hz;
    IMM32(R1,2*FSPI0CLK_Hz);
    udiv32();

    R1 = [SP++];
    unlink;

    rts;
spi_calc_baud.end:


/****************************************************************************
 Function:              spi_disable
 Description:           generic SPI disable function
 Input Parameters:      none
 Return Parameters:     none
 Registers Used:        R7,P5
 Global Registers Used: none
 C-Callable :           no
*****************************************************************************/

spi_disable:

    link 0;
    [--SP] = (R7:7,P5:5);
    IMM32(P5,SYS_MMR_BASE);


    R7 = 0(z);
    w[P5 + lo(SPI0_CTL)] = R7;
    w[P5 + lo(SPI0_BAUD)] = R7;

    R7 =       nSPIF    |   /* SPI Finished (RO) */
                MODF    |   /* Mode Fault Error (W1C) */
                TXE     |   /* Transmission Error (W1C) */
               nTXS     |   /* TDBR Data Buffer Status (RO) */
                RBSY    |   /* Receive Error (W1C) */
               nRXS     |   /* RDBR Data Buffer Status (RO) */
                TXCOL   (z);/* Transmit Collision Error (W1C) */
    w[P5 + lo(SPI0_STAT)] = R7; /* W1C: clear status bits */

    R7 = w[P5 + lo(SPI0_RDBR)] (z); /* dummy read to clear RXS bit */


    (R7:7,P5:5) = [SP++];
    unlink;

    rts;
spi_disable.end:


/*****************************************************************************
 EOF
******************************************************************************/
